using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.IO;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;

namespace WebDAVServer.CalDav.CalDav
{
    /// <summary>
    /// Folder that contains user folders which contain calendars.
    /// </summary>
    /// <remarks>
    /// This class represents /calendars/ folder. It contains user folders, such as /calendars/[user_name]/. 
    /// </remarks>
    public class CalendarsRootFolder : DavFolder
    {
        public static readonly string CalendarsRootFolderName = "calendars";

        /// <summary>
        /// Returns calendars root folder that corresponds to path.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        /// <returns>CalendarsRootFolder instance or null if not found.</returns>
        public static CalendarsRootFolder GetCalendarsRootFolder(DavContext context, string path)
        {
            if (!path.Equals(CalendarsRootFolderName, StringComparison.OrdinalIgnoreCase))
                return null;

            DirectoryInfo folder = new DirectoryInfo(context.MapPath(path));
            if (!folder.Exists)
                return null;

            return new CalendarsRootFolder(folder, context, path);
        }

        private CalendarsRootFolder(DirectoryInfo directory, DavContext context, string path)
            : base(directory, context, path)
        {
        }


        /* If required you can appy some rules, for example prohibit creating files in this folder

        /// <summary>
        /// Prohibit creating files in this folder.
        /// </summary>
        override public IFile CreateFile(string name)
        {
            throw new DavException("Creating files in this folder is not implemented.", DavStatus.NOT_IMPLEMENTED);
        }

        /// <summary>
        /// Prohibit creating folders via WebDAV in this folder.
        /// </summary>
        /// <remarks>
        /// New user folders are created during first login.
        /// </remarks>        
        override public void CreateFolder(string name)
        {
            throw new DavException("Creating sub-folders in this folder is not implemented.", DavStatus.NOT_IMPLEMENTED);
        }
         
        /// <summary>
        /// Prohibit copying this folder.
        /// </summary>        
        override public void CopyTo(IItemCollection destFolder, string destName, bool deep, MultistatusException multistatus)
        {
            throw new DavException("Copying this folder is not allowed.", DavStatus.NOT_ALLOWED);
        }
        */

        /// <summary>
        /// Prohibit moving or renaming this folder
        /// </summary>        
        override public void MoveTo(IItemCollection destFolder, string destName, MultistatusException multistatus)
        {
            throw new DavException("Moving or renaming this folder is not allowed.", DavStatus.NOT_ALLOWED);
        }

        /// <summary>
        /// Prohibit deleting this folder.
        /// </summary>        
        override public void Delete(MultistatusException multistatus)
        {
            throw new DavException("Deleting this folder is not allowed.", DavStatus.NOT_ALLOWED);
        }
    }
}
