
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Acl;
using ITHit.WebDAV.Server.Class1;
using WebDAVServer.CalDav.Acl;
using WebDAVServer.CalDav.NtfsStreamsEngine;
using ITHit.WebDAV.Server.Search;

namespace WebDAVServer.CalDav
{
    /// <summary>
    /// Folder in WebDAV repository.
    /// </summary>
    public class DavFolder : DavHierarchyItem, IFolder
    {

        /// <summary>
        /// Corresponding instance of <see cref="DirectoryInfo"/>.
        /// </summary>
        private readonly DirectoryInfo dirInfo;

        /// <summary>
        /// Returns folder that corresponds to path.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        /// <returns>DavFolder instance or null if physical folder not found in file system.</returns>
        public static DavFolder GetFolder(DavContext context, string path)
        {
            string folderPath = context.MapPath(path).TrimEnd('\\');
            DirectoryInfo folder = new DirectoryInfo(folderPath);
            // to block vulnerability when "%20" folder can be injected into path and folder.Exists returns 'true'
            if (!folder.Exists || String.Compare(folder.FullName.TrimEnd('\\'), folderPath, StringComparison.OrdinalIgnoreCase) != 0)
                return null;

            return new DavFolder(folder, context, path);
        }

        /// <summary>
        /// Initializes a new instance of the DavFolder class.
        /// </summary>
        /// <param name="directory">Instance of <see cref="DirectoryInfo"/> class with information about the folder in file system.</param>
        /// <param name="context">Instance of <see cref="DavContext"/>.</param>
        /// <param name="path">Relative to WebDAV root folder path.</param>
        protected DavFolder(DirectoryInfo directory, DavContext context, string path)
            : base(directory, context, path.TrimEnd('/') + "/")
        {
            dirInfo = directory;
        }

        /// <summary>
        /// Retrieves children of this folder.
        /// </summary>
        /// <param name="propNames">List of properties to retrieve with the children. They will be queried by the engine later.</param>
        /// <returns>Children of the folder.</returns>
        public virtual IEnumerable<IHierarchyItem> GetChildren(IList<PropertyName> propNames)
        {
            // Enumerates all child files and folders.
            // You can filter children items in this implementation and 
            // return only items that you want to be visible for this 
            // particular user.

            FileSystemInfo[] fileInfos =
                context.FileOperation(
                    this,
                    () => dirInfo.GetFileSystemInfos(),
                    Privilege.Read);

            foreach (FileSystemInfo fileInfo in fileInfos)
            {
                string childPath = Path + EncodeUtil.EncodeUrlPart(fileInfo.Name);
                IHierarchyItem child = context.GetHierarchyItem(childPath);
                if (child != null)
                {
                    yield return child;
                }
            }
            // At the upper level we have additional folder named 'Acl' which stores users and groups.
            // This is a 'virtual' folder, it does not exist in file system.
            if (GetParent() == null)
            {
                yield return new AclFolder(context);
            }
        }

        /// <summary>
        /// Creates file with name <paramref name="name"/> in this folder.
        /// </summary>
        /// <param name="name">Name of the file.</param>
        /// <returns>The file just created.</returns>
        public virtual IFile CreateFile(string name)
        {
            string fileName = System.IO.Path.Combine(fileSystemInfo.FullName, name);
            
            context.FileOperation(
                this,
                () => new FileStream(fileName, FileMode.CreateNew).Dispose(),
                Privilege.Bind);

            return (IFile)context.GetHierarchyItem(Path + EncodeUtil.EncodeUrlPart(name));
        }

        /// <summary>
        /// Create folder with specified name in the folder.
        /// </summary>
        /// <param name="name">Name of the folder.</param>
        virtual public void CreateFolder(string name)
        {
            context.FileOperation(
                this,
                () => dirInfo.CreateSubdirectory(name),
                Privilege.Bind);
        }

        /// <summary>
        /// Delete this item.
        /// </summary>
        /// <param name="multistatus"><see cref="MultistatusException"/> to populate with child files and folders failed to delete.</param>
        public override void Delete(MultistatusException multistatus)
        {
            if (GetParent() == null)
            {
                throw new DavException("Cannot delete root.", DavStatus.ACCESS_DENIED);
            }
            bool allChildrenDeleted = true;
            foreach (IHierarchyItem child in GetChildren(new PropertyName[0]))
            {
                try
                {
                    child.Delete(multistatus);
                }
                catch (DavException ex)
                {
                    //continue the operation if a child failed to delete. Tell client about it by adding to multistatus.
                    multistatus.AddInnerException(child.Path, ex);
                    allChildrenDeleted = false;
                }
            }

            if (allChildrenDeleted)
            {
                context.FileOperation(
                    this,
                    () => dirInfo.Delete(),
                    Privilege.Unbind);
            }
        }
        
        /// <summary>
        /// Move this folder to folder <paramref name="destFolder"/>.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name for this folder at destination.</param>
        /// <param name="multistatus">Instance of <see cref="MultistatusException"/>
        /// to fill with errors ocurred while moving child items.</param>
        public override void MoveTo(IItemCollection destFolder, string destName, MultistatusException multistatus)
        {
            var targetFolder = destFolder as DavFolder;
            if (targetFolder == null)
            {
                throw new DavException("Target folder doesn't exist", DavStatus.CONFLICT);
            }

            if (isRecursive(targetFolder))
            {
                throw new DavException("Cannot move folder to its subtree.", DavStatus.FORBIDDEN);
            }

            string newDirPath = System.IO.Path.Combine(targetFolder.GetFullPath(), destName);
            string targetPath = targetFolder.Path + EncodeUtil.EncodeUrlPart(destName);

            try
            {
                //remove item with the same name at destination if it exists.
                IHierarchyItem item = context.GetHierarchyItem(targetPath);
                if (item != null)
                    item.Delete(multistatus);

                targetFolder.CreateFolder(destName);
            }
            catch (DavException ex)
            {
                //Continue the operation but report error with destination path to client.
                multistatus.AddInnerException(targetPath, ex);
                return;
            }

            //Move child items
            bool movedAll = true;
            IFolder createdFolder = (IFolder)context.GetHierarchyItem(targetPath);
            foreach (DavHierarchyItem item in GetChildren(new PropertyName[0]))
            {
                try
                {
                    item.MoveTo(createdFolder, item.Name, multistatus);
                }
                catch (DavException ex)
                {
                    //continue the operation but report error with child item to client.
                    multistatus.AddInnerException(item.Path, ex);
                    movedAll = false;
                }
            }

            if (movedAll)
            {
                Delete(multistatus);
            }
        }

        /// <summary>
        /// Copies the folder to another folder.
        /// </summary>
        /// <param name="destFolder">Destination parent folder.</param>
        /// <param name="destName">Destination folder name.</param>
        /// <param name="deep">Whether child items shall be copied.</param>
        /// <param name="multistatus">Instance of <see cref="MultistatusException"/>
        /// to fill with errors ocurred while moving child items.</param>
        public override void CopyTo(
            IItemCollection destFolder,
            string destName,
            bool deep,
            MultistatusException multistatus)
        {
            var targetFolder = destFolder as DavFolder;
            if (targetFolder == null)
            {
                throw new DavException("Target folder doesn't exist", DavStatus.CONFLICT);
            }

            if (isRecursive(targetFolder))
            {
                throw new DavException("Cannot copy to subfolder", DavStatus.FORBIDDEN);
            }

            string newDirLocalPath = System.IO.Path.Combine(targetFolder.GetFullPath(), destName);
            string targetPath = targetFolder.Path + EncodeUtil.EncodeUrlPart(destName);

            // Create folder at destination.
            try
            {
                if (!Directory.Exists(newDirLocalPath))
                {
                    targetFolder.CreateFolder(destName);
                }
            }
            catch (DavException ex)
            {
                //continue, but report error to client for the target item.
                multistatus.AddInnerException(targetPath, ex);
            }

            //Copy children
            IFolder createdFolder = (IFolder)context.GetHierarchyItem(targetPath);
            foreach (DavHierarchyItem item in GetChildren(new PropertyName[0]))
            {
                if (!deep && item is DavFolder)
                {
                    continue;
                }

                try
                {
                    item.CopyTo(createdFolder, item.Name, deep, multistatus);
                }
                catch (DavException ex)
                {
                    // If a child item failed to copy we continue but report error to client.
                    multistatus.AddInnerException(item.Path, ex);
                }
            }
        }
        /// <summary>
        /// Find all resources in the subtree which have Owner equal to current user or group current user belongs to. 
        /// </summary>
        /// <param name="matchBy">Identifies whether to search by owner or group.</param>
        /// <param name="props">Properties requested for found items.</param>
        /// <returns>Matching resources.</returns>
        public override IEnumerable<IAclHierarchyItem> GetItemsByProperty(MatchBy matchBy, IList<PropertyName> props)
        {
            //We support only searching by owner(searching by Group is not supported).
            if (matchBy != MatchBy.Owner)
            {
                yield break;
            }

            var currentUser = context.WindowsIdentity;

            foreach (var child in GetChildren(props))
            {
                if (!(child is DavHierarchyItem))
                {
                    continue;
                }

                //return this item if it has matching owner.
                var o = (PrincipalBase)((DavHierarchyItem)child).GetOwner();
                if (o.Principal.Sid == currentUser.User)
                {
                    yield return (DavHierarchyItem)child;
                }
                else
                if (o.GetGroupMembers().Any(m => ((PrincipalBase)m).Sid == currentUser.User))
                {
                    yield return (DavHierarchyItem)child;
                }
                else
                //look for matching child items.
                if (child is DavFolder)
                {
                    foreach (var c in ((DavFolder)child).GetItemsByProperty(matchBy, props))
                    {
                        yield return c;
                    }
                }
            }
        }

        /// <summary>
        /// Serializes object <paramref name="o"/> and saves it into specified NTFS stream.
        /// </summary>
        /// <typeparam name="T">Type of object to be serialized.</typeparam>
        /// <param name="streamName">NTFS stream name.</param>
        /// <param name="o">Object to be serialized.</param>
        protected override void RewriteStream<T>(string streamName, T o)
        {
            context.FileOperation(
                this,
                () => SerializationUtils.RewriteStream(
                    streamName,
                    fileSystemInfo, o != null ? SerializationUtils.Serialize(o) : null),
                Privilege.Write);
        }

        /// <summary>
        /// Deletes NTFS stream with specified name.
        /// </summary>
        /// <param name="fileName">File name.</param>
        /// <param name="streamName">NTFS stream name.</param>
        protected override void DeleteStream(string fileName, string streamName)
        {
            new AlternateDataStreamInfo(fileName, streamName).Delete();
        }

        /// <summary>
        /// Determines whether <paramref name="destFolder"/> is inside this folder.
        /// </summary>
        /// <param name="destFolder">Folder to check.</param>
        /// <returns>Returns <c>true</c> if <paramref name="destFolder"/> is inside thid folder.</returns>
        private bool isRecursive(DavFolder destFolder)
        {
            return destFolder.Path.StartsWith(Path);
        }
    }
}
