//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.CalDav)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
using System.IO;
using System.Text.RegularExpressions;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;
using ITHit.WebDAV.Server.CalDav.iTip;
using ITHit.WebDAV.Server.CalDav.iTip.Transport;
using ITHit.WebDAV.Server.iCalendar;
using ITHit.WebDAV.Server.iCalendar.Serialization;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav
{
    /// <summary>
    /// Represents calendar file that contains definition of the calendar resource in the iCalendar format.
    /// </summary>
    /// <remarks>
    /// Calendar files correspond to the following path: /calendars/[username]/[calendarname]/[filename].ics.
    /// </remarks>
    public class CalendarFile : DavFile, ICalendarFile
    {
        private readonly IEventSchedulingNotifier eventSchedulingNotifier;

        /// <summary>
        /// Returns calendar file that corresponds to path.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        /// <returns>CalendarFile instance or null if not found.</returns>
        public static CalendarFile GetCalendarFile(DavContext context, string path)
        {
            string pattern = string.Format(@"^/?{0}/(?<user_name>[^/]+)/(?<calendar_name>[^/]+)/(?<file_name>[^/]+\.ics)$",
                              CalendarsRootFolder.CalendarsRootFolderName);
            if (!Regex.IsMatch(path, pattern))
                return null;

            FileInfo file = new FileInfo(context.MapPath(path));
            if (!file.Exists)
                return null;

            return new CalendarFile(file, context, path);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="CalendarFile"/> class.
        /// </summary>
        /// <param name="file"><see cref="FileInfo"/> for corresponding object in file system.</param>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        private CalendarFile(FileInfo file, DavContext context, string path)
            : base(file, context, path)
        {
            eventSchedulingNotifier = new EventSchedulingNotifier(new iMipEventSchedulingTransport());
        }

        public override void Delete(MultistatusException multistatus)
        {
            NotifyAttendees(null);
            base.Delete(multistatus);
        }

        public override bool Write(Stream content, string contentType, long startIndex, long totalFileSize)
        {
            bool result = base.Write(content, contentType, startIndex, totalFileSize);

            string calendarObjectContent = File.ReadAllText(fileSystemInfo.FullName);
            NotifyAttendees(calendarObjectContent);

            return result;
        }

        private void NotifyAttendees(string calendarObjectContent)
        {
            CalendarObjectSerializer serializer = new CalendarObjectSerializer();
            string oldCalendarObjectContent = SerializationUtils.GetStreamAndDeserialize("CalendarObject", fileSystemInfo, context.Logger);
            RewriteStream("CalendarObject", calendarObjectContent);
            ICalendarObject calendarObject = null;
            ICalendarObject oldCalendarObject = null;

            if (calendarObjectContent != null)
            {
                calendarObject = serializer.Deserialize(calendarObjectContent);
            }

            if (oldCalendarObjectContent != null)
            {
                //this is a workaround for now since after deserialization all \r disappear
                oldCalendarObjectContent = oldCalendarObjectContent.Replace("\n", "\r\n");
                oldCalendarObject = serializer.Deserialize(oldCalendarObjectContent);
            }

            try
            {
                eventSchedulingNotifier.NotifyAboutChanges(oldCalendarObject, calendarObject);
            }
            catch (Exception ex)
            {
                // SMTP server is not configured in web.config/app.config
                context.Logger.LogError("Faled to notify attendees about event change. SMTP server is not configured in web.config/app.config\n", ex);
            }
        }
    }
}
